searchData={"content_type":"text/markdown","items":[{"doc":"Holds the state of the config.\n\nSee other modules for more details:\n- `TelemetryUI.Theme`\n- `TelemetryUI.Backend`","ref":"TelemetryUI.Config.html","title":"TelemetryUI.Config","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"TelemetryUI.Config.html#child_spec/1","title":"TelemetryUI.Config.child_spec/1","type":"function"},{"doc":"","ref":"TelemetryUI.Config.html#start_link/1","title":"TelemetryUI.Config.start_link/1","type":"function"},{"doc":"Main entry point to start all the processes as part of your application.\n\n# Usage\n```\nuse Application\n\ndef start(_type, _args) do\n  children = [\n    Shopcast.Repo,\n    MyAppWeb.Endpoint,\n    {TelemetryUI, my_config()}\n  ]\n```\nThe reporter, write buffer and pruner processes will be part of your supervision tree, ensuring that everything runs smoothly.\n\nThe config is a data structure representing all settings used by TelemetryUI.","ref":"TelemetryUI.html","title":"TelemetryUI","type":"module"},{"doc":"```\n  [\n    metrics: [\n      {\"System\", [last_value(\"vm.memory.total\", unit: {:byte, :megabyte})]}\n    ],\n    theme: %{title: \"Metrics\"},\n    backend: \n      %TelemetryUI.Backend.EctoPostgres{\n        repo: MyApp.Repo,\n        pruner_threshold: [months: -1],\n        pruner_interval_ms: 3_600_000,\n        max_buffer_size: 1_000,\n        flush_interval_ms: 10_000\n      }\n  ]\n```\n\nThis config will show a basic \"last value\" chart for the memory usage reported by the VM.\nThe title of the page will be \"Metrics\" and the backend used to store and query metrics will be PostgreSQL.\nSee `TelemetryUI.Config` for a list of all options.","ref":"TelemetryUI.html#module-config-example","title":"Config example - TelemetryUI","type":"module"},{"doc":"Every kind of metrics exposed by `TelemetryMetrics` is supported by `TelemetryUI`:\n- `last_value`\n- `summary`\n- `sum`\n- `counter`\n- `distribution`\n\n`TelemetryUI` also exposes its own set of metrics:\n- `average`\n- `average_over_time`\n- `count_over_time`\n- `median`\n- `median_over_time`","ref":"TelemetryUI.html#module-metrics-list","title":"Metrics list - TelemetryUI","type":"module"},{"doc":"Finally, when the configuration is done and we actually want to see our metrics, we need to add the `TelemetryUI.Web` module in our router:\n\n*Phoenix*\n```\nscope \"/\" do\n  pipe_through([:browser])\n  get(\"/metrics\", TelemetryUI.Web, [], [assigns: %{telemetry_ui_allowed: true}])\nend\n```\n\nThe metrics page is protected by default. It needs to have the `telemetry_ui_allowed` assign to true to render.\nWe can imagine having a `:admin_protected` plug that ensure a user is an admin and also assign `telemetry_ui_allowed` to true.\n\n```\nscope \"/\" do\n  pipe_through([:browser, :admin_protected])\n  get(\"/metrics\", TelemetryUI.Web, [])\nend\n```\n\nThat’s it! The `/metrics` page will show the metrics as they are recorded. Checkout the Guides to dive into more complex configuration and awesome features :)","ref":"TelemetryUI.html#module-the-web","title":"The Web - TelemetryUI","type":"module"},{"doc":"","ref":"TelemetryUI.html#backend/1","title":"TelemetryUI.backend/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"TelemetryUI.html#child_spec/1","title":"TelemetryUI.child_spec/1","type":"function"},{"doc":"","ref":"TelemetryUI.html#config_name/1","title":"TelemetryUI.config_name/1","type":"function"},{"doc":"","ref":"TelemetryUI.html#insert_metric_data/2","title":"TelemetryUI.insert_metric_data/2","type":"function"},{"doc":"","ref":"TelemetryUI.html#metric_by_id/2","title":"TelemetryUI.metric_by_id/2","type":"function"},{"doc":"","ref":"TelemetryUI.html#metric_data/3","title":"TelemetryUI.metric_data/3","type":"function"},{"doc":"","ref":"TelemetryUI.html#page_by_id/2","title":"TelemetryUI.page_by_id/2","type":"function"},{"doc":"","ref":"TelemetryUI.html#page_by_title/2","title":"TelemetryUI.page_by_title/2","type":"function"},{"doc":"","ref":"TelemetryUI.html#pages/1","title":"TelemetryUI.pages/1","type":"function"},{"doc":"","ref":"TelemetryUI.html#start_link/1","title":"TelemetryUI.start_link/1","type":"function"},{"doc":"","ref":"TelemetryUI.html#theme/1","title":"TelemetryUI.theme/1","type":"function"},{"doc":"","ref":"TelemetryUI.html#valid_share_key?/1","title":"TelemetryUI.valid_share_key?/1","type":"function"},{"doc":"","ref":"TelemetryUI.html#valid_share_url?/1","title":"TelemetryUI.valid_share_url?/1","type":"function"},{"doc":"","ref":"TelemetryUI.html#writer_buffer_name/1","title":"TelemetryUI.writer_buffer_name/1","type":"function"},{"doc":"Customize the UI of the web view.\n\n- `primary_color`: *Example: #ff0000*, Used for the title and first color on the charts\n- `title`: *Example: \"My metrics\"*, Used for the HTML page title and the title in the header.\n- `description`: *Example: \"HTTP, GraphQL and Database metrics\"*, Used for the HTML description tag.\n- `logo`: SVG logo in string used for the favicon and header.\n- `scale`: List of hex colors used in the charts.\n- `share_key`: 16 characters string used to hash the shared page.\n- `share_path`: Path in string starting by a `/` used to generate the shared page. It’s difference from the metrics page since it does not require authentication.\n- `frame_options`: *Example: [{:last_30_minutes, 30, :minute}, {:last_2_hours, 120, :minute}]*, Used to generate the time frame in the header.","ref":"TelemetryUI.Theme.html","title":"TelemetryUI.Theme","type":"module"},{"doc":"Inserting, fetching and pruning of metrics.","ref":"TelemetryUI.Backend.html","title":"TelemetryUI.Backend","type":"protocol"},{"doc":"Inserting is called by the internal `WriteBuffer`. It can be grouped depending on the buffer configuration inside the backend struct:\n\n- `flush_interval_ms`: Time interval before the write buffer calls the backend\n- `max_buffer_size`: Maximum count of events before the write  buffer calls the backend","ref":"TelemetryUI.Backend.html#module-inserting","title":"Inserting - TelemetryUI.Backend","type":"protocol"},{"doc":"Fetching is called when rendering metrics in the view. The `filters` argument is a struct with predefined fields: `to`, `from`, `event_name` and `compare`.","ref":"TelemetryUI.Backend.html#module-fetching","title":"Fetching - TelemetryUI.Backend","type":"protocol"},{"doc":"Pruning is implemented to keep the datastore clean. Keeping data forever will increase the size of the storage and affect performance.\n- `pruner_threshold`: *Example:* `[months: -1]`. Delete events older than a month. \n- `pruner_interval_ms`: *Example:* 84_000. Time interval for the pruner process to run. The process simply calls `#prune_events!/2`.","ref":"TelemetryUI.Backend.html#module-pruning","title":"Pruning - TelemetryUI.Backend","type":"protocol"},{"doc":"","ref":"TelemetryUI.Backend.html#insert_event/6","title":"TelemetryUI.Backend.insert_event/6","type":"function"},{"doc":"","ref":"TelemetryUI.Backend.html#metric_data/3","title":"TelemetryUI.Backend.metric_data/3","type":"function"},{"doc":"","ref":"TelemetryUI.Backend.html#prune_events!/2","title":"TelemetryUI.Backend.prune_events!/2","type":"function"},{"doc":"","ref":"TelemetryUI.Backend.html#t:t/0","title":"TelemetryUI.Backend.t/0","type":"type"},{"doc":"Migrations create and modify the database tables TelemetryUI needs to function.","ref":"TelemetryUI.Backend.EctoPostgres.Migrations.html","title":"TelemetryUI.Backend.EctoPostgres.Migrations","type":"module"},{"doc":"To use migrations in your application you'll need to generate an `Ecto.Migration` that wraps\ncalls to `TelemetryUI.Backend.EctoPostgres.Migrations`:\n\n```bash\nmix ecto.gen.migration add_telemetry_ui\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`TelemetryUI.Backend.EctoPostgres.Migrations`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddTelemetryUI do\n  use Ecto.Migration\n\n  def up, do: TelemetryUI.Backend.EctoPostgres.Migrations.up()\n\n  def down, do: TelemetryUI.Backend.EctoPostgres.Migrations.down()\nend\n```\n\nThis will run all of TelemetryUI's versioned migrations for your database.\n\nNow, run the migration to create the table:\n\n```bash\nmix ecto.migrate\n```\n\nMigrations between versions are idempotent. As new versions are released, you\nmay need to run additional migrations. To do this, generate a new migration:\n\n```bash\nmix ecto.gen.migration upgrade_telemetry_ui_to_v11\n```\n\nOpen the generated migration in your editor and call the `up` and `down`\nfunctions on `TelemetryUI.Backend.EctoPostgres.Migrations`, passing a version number:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.UpgradeTelemetryUIToV11 do\n  use Ecto.Migration\n\n  def up, do: TelemetryUI.Backend.EctoPostgres.Migrations.up(version: 11)\n\n  def down, do: TelemetryUI.Backend.EctoPostgres.Migrations.down(version: 11)\nend\n```","ref":"TelemetryUI.Backend.EctoPostgres.Migrations.html#module-usage","title":"Usage - TelemetryUI.Backend.EctoPostgres.Migrations","type":"module"},{"doc":"TelemetryUI supports namespacing through PostgreSQL schemas, also called \"prefixes\" in Ecto. With\nprefixes your events table can reside outside of your primary schema (usually public) and you can\nhave multiple separate job tables.\n\nTo use a prefix you first have to specify it within your migration:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedTelemetryUIJobsTable do\n  use Ecto.Migration\n\n  def up, do: TelemetryUI.Backend.EctoPostgres.Migrations.up(prefix: \"private\")\n\n  def down, do: TelemetryUI.Backend.EctoPostgres.Migrations.down(prefix: \"private\")\nend\n```\n\nThe migration will create the \"private\" schema and all tables, functions and triggers within\nthat schema. With the database migrated you'll then specify the prefix in your configuration:\n\n```elixir\nconfig :my_app, TelemetryUI,\n  prefix: \"private\",\n  ...\n```\n\nIn some cases, for example if your \"private\" schema already exists and your database user in\nproduction doesn't have permissions to create a new schema, trying to create the schema from the\nmigration will result in an error. In such situations, it may be useful to inhibit the creation\nof the \"private\" schema:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedTelemetryUIJobsTable do\n  use Ecto.Migration\n\n  def up, do: TelemetryUI.Backend.EctoPostgres.Migrations.up(prefix: \"private\", create_schema: false)\n\n  def down, do: TelemetryUI.Backend.EctoPostgres.Migrations.down(prefix: \"private\")\nend\n```","ref":"TelemetryUI.Backend.EctoPostgres.Migrations.html#module-isolation-with-prefixes","title":"Isolation with Prefixes - TelemetryUI.Backend.EctoPostgres.Migrations","type":"module"},{"doc":"Run the `down` changes for all migrations between the current version and the initial version.","ref":"TelemetryUI.Backend.EctoPostgres.Migrations.html#down/1","title":"TelemetryUI.Backend.EctoPostgres.Migrations.down/1","type":"function"},{"doc":"Run all migrations from current version down to the first:\n\n    TelemetryUI.Backend.EctoPostgres.Migrations.down()\n\nRun migrations down to and including a specified version:\n\n    TelemetryUI.Backend.EctoPostgres.Migrations.down(version: 5)\n\nRun migrations in an alternate prefix:\n\n    TelemetryUI.Backend.EctoPostgres.Migrations.down(prefix: \"payments\")","ref":"TelemetryUI.Backend.EctoPostgres.Migrations.html#down/1-example","title":"Example - TelemetryUI.Backend.EctoPostgres.Migrations.down/1","type":"function"},{"doc":"Run the `up` changes for all migrations between the initial version and the current version.","ref":"TelemetryUI.Backend.EctoPostgres.Migrations.html#up/1","title":"TelemetryUI.Backend.EctoPostgres.Migrations.up/1","type":"function"},{"doc":"Run all migrations up to the current version:\n\n    TelemetryUI.Backend.EctoPostgres.Migrations.up()\n\nRun migrations up to a specified version:\n\n    TelemetryUI.Backend.EctoPostgres.Migrations.up(version: 2)\n\nRun migrations in an alternate prefix:\n\n    TelemetryUI.Backend.EctoPostgres.Migrations.up(prefix: \"payments\")\n\nRun migrations in an alternate prefix but don't try to create the schema:\n\n    TelemetryUI.Backend.EctoPostgres.Migrations.up(prefix: \"payments\", create_schema: false)","ref":"TelemetryUI.Backend.EctoPostgres.Migrations.html#up/1-example","title":"Example - TelemetryUI.Backend.EctoPostgres.Migrations.up/1","type":"function"},{"doc":"Plug to render an HTML view with all metrics.\nThe view handles the different pages in the configurationa and the assets pipeline for CSS and JavaScript.\nThe module also handles \"async\" components data request called in the components.","ref":"TelemetryUI.Web.html","title":"TelemetryUI.Web","type":"module"},{"doc":"","ref":"TelemetryUI.Web.html#call/2","title":"TelemetryUI.Web.call/2","type":"function"},{"doc":"","ref":"TelemetryUI.Web.html#index/2","title":"TelemetryUI.Web.index/2","type":"function"},{"doc":"","ref":"TelemetryUI.Web.html#init/1","title":"TelemetryUI.Web.init/1","type":"function"},{"doc":"Telemetry-based metrics UI. Take your   telemetry   metrics and display them in a web page.","ref":"readme.html","title":"README","type":"extras"},{"doc":"`TelemetryUI`’s primary goal is to display [your application metrics](https://hexdocs.pm/telemetry_metrics) without external infrastructure dependencies. [Plug](https://hexdocs.pm/plug/Plug.Telemetry.html), [Phoenix](https://hexdocs.pm/phoenix/telemetry.html), [Phoenix LiveView](https://hexdocs.pm/phoenix_live_view/telemetry.html), [Absinthe](https://hexdocs.pm/absinthe/telemetry.html), [Ecto](https://hexdocs.pm/ecto/Ecto.Repo.html#module-telemetry-events), [Erlang VM](https://hexdocs.pm/telemetry_poller/readme.html), [Tesla](https://hexdocs.pm/tesla/Tesla.Middleware.Telemetry.html), [Finch](https://hexdocs.pm/finch/Finch.Telemetry.html), [Redix](https://hexdocs.pm/redix/telemetry.html), [Oban](https://hexdocs.pm/oban/Oban.Telemetry.html), [Broadway](https://hexdocs.pm/broadway/Broadway.html#module-telemetry) and others expose all sorts of data that can be useful. You can also emit your own events from your application.\n\nYour data should not have to be uploaded somewhere else to have insighful metrics.\n\nIt comes with a Postgres backend, powered by [Ecto](https://hexdocs.pm/ecto), to quickly (and efficiently) store and query your application events.","ref":"readme.html#features","title":"Features - README","type":"extras"},{"doc":"- Persisted metrics inside your own database\n- Live dashboard\n- Many built-in charts and visualizations using [VegaLite](https://vega.github.io/vega-lite/)","ref":"readme.html#advantages-over-other-tools","title":"Advantages over other tools - README","type":"extras"},{"doc":"- 100% custom UI hook to show your own components\n- 100% custom data fetching to show live data\n- Shareable metrics page (secured, cacheable, without external requests)\n- Slack digest with rendered images\n- Multiple metrics dashboard living in the same app\n\nCheckout the Guides for more informations.","ref":"readme.html#advanced-features","title":"Advanced features - README","type":"extras"},{"doc":"","ref":"readme.html#usage","title":"Usage - README","type":"extras"},{"doc":"TelemetryUI is published on Hex. Add it to your list of dependencies in `mix.exs`:\n\n```elixir\n# mix.exs\ndef deps do\n  [\n    {:telemetry_ui, \"~> 4.0\"}\n  ]\nend\n```\n\nConfigure TelemetryUI for test.\n\n```elixir\n# config/test.exs\nconfig :telemetry_ui, disabled: true\n```\n\nThen run mix deps.get to install Telemetry and its dependencies.\n\nAfter the packages are installed you must create a database migration to add the `telemetry_ui_events` table to your database:\n\n```sh\nmix ecto.gen.migration add_telemetry_ui_events_table\n```\n\nOpen the generated migration in your editor and call the up and down functions on `TelemetryUI.Adapter.EctoPostgres.Migrations`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddTelemetryUIEventsTable do\n  use Ecto.Migration\n\n  @disable_migration_lock true\n  @disable_ddl_transaction true\n\n  def up do\n    TelemetryUI.Backend.EctoPostgres.Migrations.up()\n  end\n\n  # We specify `version: 1` in `down`, ensuring that we'll roll all the way back down if\n  # necessary, regardless of which version we've migrated `up` to.\n  def down do\n    TelemetryUI.Backend.EctoPostgres.Migrations.down(version: 1)\n  end\nend\n```\n\nThis will run all of TelemetryUI's versioned migrations for your database. Migrations between versions are idempotent and rarely change after a release. As new versions are released you may need to run additional migrations.\n\nNow, run the migration to create the table:\n\n```sh\nmix ecto.migrate\n```\n\nTelemetryUI instances are isolated supervision trees and must be included in your application's supervisor to run. Use the application configuration you've just set and include TelemetryUI in the list of supervised children:\n\n```elixir\n# lib/my_app/application.ex\ndef start(_type, _args) do\n  children = [\n    MyApp.Repo,\n    {TelemetryUI, telemetry_config()}\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\nend\n\ndefp telemetry_config do\n  import TelemetryUI.Metrics\n\n  [\n    metrics: [\n      last_value(\"my_app.users.total_count\", description: \"Number of users\", ui_options: [unit: \" users\"]),\n      counter(\"phoenix.router_dispatch.stop.duration\", description: \"Number of requests\", unit: {:native, :millisecond}, ui_options: [unit: \" requests\"]),\n      value_over_time(\"vm.memory.total\", unit: {:byte, :megabyte}),\n      distribution(\"phoenix.router_dispatch.stop.duration\", description: \"Requests duration\", unit: {:native, :millisecond}, reporter_options: [buckets: [0, 100, 500, 2000]]),\n    ],\n    backend: %TelemetryUI.Backend.EctoPostgres{\n      repo: MyApp.Repo,\n      pruner_threshold: [months: -1],\n      pruner_interval_ms: 84_000,\n      max_buffer_size: 10_000,\n      flush_interval_ms: 10_000\n    }\n  ]\nend\n```\n\nSince the config is read once at startup, you need to restart the server of you add new metrics to track.\n\nTo see the rendered metrics, you need to add a route to your router.\n\n```elixir\n# lib/my_app_web/router.ex\nscope \"/\" do\n  get(\"/metrics\", TelemetryUI.Web, [], [assigns: %{telemetry_ui_allowed: true}])\nend\n```","ref":"readme.html#installation","title":"Installation - README","type":"extras"},{"doc":"Optionally, you can add 2 dependencies to generate png images from the charts. When you use the share feature, every chart will have an associated image (hidden) that will render a nice preview when sharing on Slack/other platforms.\n\n```elixir\n# mix.exs\ndef deps do\n  [\n    #...\n    {:vix, \"~> 0.26\"},\n    {:vega_lite_convert, \"~> 0.6\"},\n  ]\nend\n```\n\n#### Security\n\nSince it may contain sensitive data, `TelemetryUI` requires a special assign to render the page.\n\n`:telemetry_ui_allowed` must be set to `true` in the `conn` struct before it enters the `TelemetryUI.Web` module.\n\nBy using a special assign to control access, you can integrate `TelemetryUI.Web` page with your existing authorization. We can imagine an admin protected section that also gives you access to the `TelemetryUI.Web` page:\n\n```elixir\npipeline :admin_protected do\n  plug(MyAppWeb.EnsureCurrentUser)\n  plug(MyAppWeb.EnsureRole, :admin)\n  plug(:enable_telemetry_ui)\nend\n\ndef enable_telemetry_ui(conn, _), do: assign(conn, :telemetry_ui_allowed, true)\n```\n\nThat’s it! You can declare as many metrics as you want and they will render in HTML on your page!","ref":"readme.html#image-generation","title":"Image generation - README","type":"extras"},{"doc":"`TelemetryUI` is © 2024 [Mirego](https://www.mirego.com) and may be freely distributed under the [New BSD license](http://opensource.org/licenses/BSD-3-Clause). See the [`LICENSE.md`](https://github.com/mirego/telemetry_ui/blob/master/LICENSE.md) file.","ref":"readme.html#license","title":"License - README","type":"extras"},{"doc":"[Mirego](https://www.mirego.com) is a team of passionate people who believe that work is a place where you can innovate and have fun. We’re a team of [talented people](https://life.mirego.com) who imagine and build beautiful Web and mobile applications. We come together to share ideas and [change the world](http://www.mirego.org).\n\nWe also [love open-source software](https://open.mirego.com) and we try to give back to the community as much as we can.","ref":"readme.html#about-mirego","title":"About Mirego - README","type":"extras"},{"doc":"# Using application data\n\nSometimes we already have metrics that we want to display without going through the `telemetry` package.\nWe want to expose the number of users created. The `users` table with the `inserted_at` is perfect for this.\n`telemetry_ui` provides a way use the existing metrics helpers with application-defined data:\n\n```elixir\ncount_over_time(:data,\n  description: \"Users count\",\n  unit: \" users\",\n  data_resolver: fn options ->\n    query =\n      from(\n        users in \"users\",\n        select: %{date: users.inserted_at, count: 1},\n        where: users.inserted_at >= ^options.from and users.inserted_at <= ^options.to\n      )\n\n    {:ok, MyApp.Repo.all(query)}\n  end\n)\n```\n\nThe `data_resolver` options is specified here to return the data that will be displayed in the same graph layout as the `telemetry_ui_events` entries.\nThe built-in metrics uses the same `data_resolver` options: `data_resolver: &{:async, fn -> TelemetryUI.metric_data(&1, metric, &2) end}`.","ref":"application-data.html","title":"Using application data","type":"extras"},{"doc":"# Custom UI\n\n`telemetry_ui` offers many way to customize the layouts, colors and components of your dashboard.","ref":"custom-ui.html","title":"Custom UI","type":"extras"},{"doc":"The `theme` option of you configuration can be customized:\n\n```elixir\n[\n  metrics: [\n    {\"Users\", [count_over_time(\"myapp.users.created\")]}\n  ],\n  backend: backend(),\n  theme: %{\n    header_color: \"blue\",\n    title: \"User dashboard metrics\"\n  }\n]\n```\n\nHere is the list of available options:\n\n- `header_color`: A CSS color used as the text color in the header\n- `title`: The title used in the header\n- `logo`: An SVG binary used in the header and the favicon\n- `scale`: Available colors used in graph. The colors are used in the same order they are defined in.\n- `share_key`: 16 characters key that enable the sharing of a dashboard page. The share feature is hidden when the option is `nil`\n- `share_path`: Application router path to render the share pages.\n- `frame_options`: List of options for the time frame select in the UI. Format: `{atom_identitifer, number, unit}` `{:last_3_minutes, 3, :minute}`","ref":"custom-ui.html#theme","title":"Theme - Custom UI","type":"extras"},{"doc":"The list of metrics can also be customized with TailwindCSS classes. Here is an example with 2 metrics side-by-side in a grid:\n\n```elixir\nmetrics = [\n  distribution(\"my_app.repo.query.total_time\",\n    reporter_options: [buckets: [0, 4, 10, 50]],\n    ui_options: [class: \"col-span-2\"],\n    unit: {:native, :millisecond}\n  ),\n  distribution(\"phoenix.router_dispatch.stop.duration\",\n    reporter_options: [buckets: [0, 40, 100, 500]],\n    ui_options: [class: \"col-span-2\"],\n    unit: {:native, :millisecond}\n  ),\n  distribution(\"phoenix.router_dispatch.stop.duration\",\n    tags: [:route],\n    reporter_options: [buckets: [0, 40, 100, 500]],\n    ui_options: [class: \"col-span-full\"],\n    unit: {:native, :millisecond}\n  )\n]\n\n[\n  metrics: [\n    {\"Metrics layout page\", metrics, ui_options: [metrics_class: \"grid-cols-4 gap-4\"]}\n  ],\n  backend: #...\n  theme: #...\n]\n```\n\nAll grid classes are available with `sm` and `lg` variant to support mobile and desktop viewports.","ref":"custom-ui.html#layout","title":"Layout - Custom UI","type":"extras"},{"doc":"# Multiple metrics endpoints\n\nThe `/metrics` endpoint exposes different pages of metrics. We can also have multiple metrics endpoint.\nThe benefit is to have different metrics exposed with different permissions. Here is an example:\n\nWe have our router defining routes with different `telemetry_ui_name` Plug.Conn assign.\n\n```elixir\nscope \"/admin\" do\n  pipe_through [EnsureAdmin]\n\n  get(\"/metrics\", TelemetryUI.Web, :index, assigns: %{telemetry_ui_name: :admin})\nend\n\nscope \"/user_dashboard\" do\n  pipe_through [EnsureUser]\n\n  get(\"/metrics\", TelemetryUI.Web, :index, assigns: %{telemetry_ui_name: :user_dashboard})\nend\n```\n\n```elixir\ndefmodule MyApp.Telemetry do\n  import TelemetryUI.Metrics\n\n  def admin do\n    [\n      name: :admin,\n      metrics: [\n        {\"Memory\", [value_over_time(\"vm.memory.total\", unit: {:byte, :megabyte})]}\n      ],\n      backend: backend(),\n      theme: %{header_color: \"purple\", title: \"Admin metrics\"}\n    ]\n  end\n\n  def user_dashboard do\n    [\n      name: :user_dashboard,\n      metrics: [\n        {\"Followers\", [count_over_time(\"myapp.users.new_follower\")]}\n      ],\n      backend: backend(),\n      theme: %{header_color: \"blue\", title: \"User dashboard metrics\"}\n    ]\n  end\n\n  defp backend do\n    %TelemetryUI.Backend.EctoPostgres{\n      repo: MyApp.Repo,\n      pruner_threshold: [months: -1],\n      pruner_interval_ms: 84_000,\n      max_buffer_size: 1,\n      flush_interval_ms: 1,\n      verbose: true\n    }\n  end\nend\n```\n\nDon’t forget to start all `TelemetryUI` process in `application.ex`:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # ...\n      {TelemetryUI, MyApp.Telemetry.admin()},\n      {TelemetryUI, MyApp.Telemetry.user_dashboard()}\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"multi-metrics-endpoints.html","title":"Multiple metrics endpoints","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}